# Copyright 2025-2026 Dimensional Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import ABC, abstractmethod


class Resource(ABC):
    @abstractmethod
    def start(self) -> None: ...

    @abstractmethod
    def stop(self) -> None: ...

    def dispose(self) -> None:
        """
        Makes a Resource disposable
        So you can do a

        from reactivex.disposable import CompositeDisposable

        disposables = CompositeDisposable()

        transport1 = LCMTransport(...)
        transport2 = LCMTransport(...)

        disposables.add(transport1)
        disposables.add(transport2)

        ...

        disposables.dispose()

        """
        self.stop()
