# Copyright 2025-2026 Dimensional Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
from numpy.typing import NDArray

def min_cost_astar_cpp(
    grid: NDArray[np.int8],
    start_x: int,
    start_y: int,
    goal_x: int,
    goal_y: int,
    cost_threshold: int,
    unknown_penalty: float,
) -> list[tuple[int, int]]: ...
