# Copyright 2025 Dimensional Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Simulation manipulator utilities."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from dimos.simulation.manipulators.sim_manip_interface import SimManipInterface
    from dimos.simulation.manipulators.sim_module import (
        SimulationModule,
        SimulationModuleConfig,
        simulation,
    )

__all__ = [
    "SimManipInterface",
    "SimulationModule",
    "SimulationModuleConfig",
    "simulation",
]


def __getattr__(name: str):  # type: ignore[no-untyped-def]
    if name == "SimManipInterface":
        from dimos.simulation.manipulators.sim_manip_interface import SimManipInterface

        return SimManipInterface
    if name in {"SimulationModule", "SimulationModuleConfig", "simulation"}:
        from dimos.simulation.manipulators.sim_module import (
            SimulationModule,
            SimulationModuleConfig,
            simulation,
        )

        return {
            "SimulationModule": SimulationModule,
            "SimulationModuleConfig": SimulationModuleConfig,
            "simulation": simulation,
        }[name]
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
