# Copyright 2025-2026 Dimensional Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This file is auto-generated. Do not edit manually.
# Run `pytest dimos/robot/test_all_blueprints_generation.py` to regenerate.

all_blueprints = {
    "arm-teleop": "dimos.teleop.quest.blueprints:arm_teleop",
    "arm-teleop-dual": "dimos.teleop.quest.blueprints:arm_teleop_dual",
    "arm-teleop-piper": "dimos.teleop.quest.blueprints:arm_teleop_piper",
    "arm-teleop-visualizing": "dimos.teleop.quest.blueprints:arm_teleop_visualizing",
    "arm-teleop-xarm6": "dimos.teleop.quest.blueprints:arm_teleop_xarm6",
    "coordinator-basic": "dimos.control.blueprints:coordinator_basic",
    "coordinator-cartesian-ik-mock": "dimos.control.blueprints:coordinator_cartesian_ik_mock",
    "coordinator-cartesian-ik-piper": "dimos.control.blueprints:coordinator_cartesian_ik_piper",
    "coordinator-combined-xarm6": "dimos.control.blueprints:coordinator_combined_xarm6",
    "coordinator-dual-mock": "dimos.control.blueprints:coordinator_dual_mock",
    "coordinator-dual-xarm": "dimos.control.blueprints:coordinator_dual_xarm",
    "coordinator-mock": "dimos.control.blueprints:coordinator_mock",
    "coordinator-piper": "dimos.control.blueprints:coordinator_piper",
    "coordinator-piper-xarm": "dimos.control.blueprints:coordinator_piper_xarm",
    "coordinator-teleop-dual": "dimos.control.blueprints:coordinator_teleop_dual",
    "coordinator-teleop-piper": "dimos.control.blueprints:coordinator_teleop_piper",
    "coordinator-teleop-xarm6": "dimos.control.blueprints:coordinator_teleop_xarm6",
    "coordinator-velocity-xarm6": "dimos.control.blueprints:coordinator_velocity_xarm6",
    "coordinator-xarm6": "dimos.control.blueprints:coordinator_xarm6",
    "coordinator-xarm7": "dimos.control.blueprints:coordinator_xarm7",
    "demo-agent": "dimos.agents.demo_agent:demo_agent",
    "demo-agent-camera": "dimos.agents.demo_agent:demo_agent_camera",
    "demo-camera": "dimos.hardware.sensors.camera.module:demo_camera",
    "demo-error-on-name-conflicts": "dimos.robot.unitree.demo_error_on_name_conflicts:demo_error_on_name_conflicts",
    "demo-google-maps-skill": "dimos.agents.skills.demo_google_maps_skill:demo_google_maps_skill",
    "demo-gps-nav": "dimos.agents.skills.demo_gps_nav:demo_gps_nav",
    "demo-grasping": "dimos.manipulation.grasping.demo_grasping:demo_grasping",
    "demo-object-scene-registration": "dimos.perception.demo_object_scene_registration:demo_object_scene_registration",
    "demo-osm": "dimos.mapping.osm.demo_osm:demo_osm",
    "demo-skill": "dimos.agents.skills.demo_skill:demo_skill",
    "dual-xarm6-planner": "dimos.manipulation.manipulation_blueprints:dual_xarm6_planner",
    "keyboard-teleop-piper": "dimos.robot.manipulators.piper.blueprints:keyboard_teleop_piper",
    "keyboard-teleop-xarm6": "dimos.robot.manipulators.xarm.blueprints:keyboard_teleop_xarm6",
    "keyboard-teleop-xarm7": "dimos.robot.manipulators.xarm.blueprints:keyboard_teleop_xarm7",
    "mid360": "dimos.hardware.sensors.lidar.livox.livox_blueprints:mid360",
    "mid360-fastlio": "dimos.hardware.sensors.lidar.fastlio2.fastlio_blueprints:mid360_fastlio",
    "mid360-fastlio-voxels": "dimos.hardware.sensors.lidar.fastlio2.fastlio_blueprints:mid360_fastlio_voxels",
    "mid360-fastlio-voxels-native": "dimos.hardware.sensors.lidar.fastlio2.fastlio_blueprints:mid360_fastlio_voxels_native",
    "phone-go2-teleop": "dimos.teleop.phone.blueprints:phone_go2_teleop",
    "simple-phone-teleop": "dimos.teleop.phone.blueprints:simple_phone_teleop",
    "uintree-g1-primitive-no-nav": "dimos.robot.unitree.g1.blueprints.primitive.uintree_g1_primitive_no_nav:uintree_g1_primitive_no_nav",
    "unitree-g1": "dimos.robot.unitree.g1.blueprints.perceptive.unitree_g1:unitree_g1",
    "unitree-g1-agentic": "dimos.robot.unitree.g1.blueprints.agentic.unitree_g1_agentic:unitree_g1_agentic",
    "unitree-g1-agentic-sim": "dimos.robot.unitree.g1.blueprints.agentic.unitree_g1_agentic_sim:unitree_g1_agentic_sim",
    "unitree-g1-basic": "dimos.robot.unitree.g1.blueprints.basic.unitree_g1_basic:unitree_g1_basic",
    "unitree-g1-basic-sim": "dimos.robot.unitree.g1.blueprints.basic.unitree_g1_basic_sim:unitree_g1_basic_sim",
    "unitree-g1-detection": "dimos.robot.unitree.g1.blueprints.perceptive.unitree_g1_detection:unitree_g1_detection",
    "unitree-g1-full": "dimos.robot.unitree.g1.blueprints.agentic.unitree_g1_full:unitree_g1_full",
    "unitree-g1-joystick": "dimos.robot.unitree.g1.blueprints.basic.unitree_g1_joystick:unitree_g1_joystick",
    "unitree-g1-shm": "dimos.robot.unitree.g1.blueprints.perceptive.unitree_g1_shm:unitree_g1_shm",
    "unitree-g1-sim": "dimos.robot.unitree.g1.blueprints.perceptive.unitree_g1_sim:unitree_g1_sim",
    "unitree-go2": "dimos.robot.unitree.go2.blueprints.smart.unitree_go2:unitree_go2",
    "unitree-go2-agentic": "dimos.robot.unitree.go2.blueprints.agentic.unitree_go2_agentic:unitree_go2_agentic",
    "unitree-go2-agentic-huggingface": "dimos.robot.unitree.go2.blueprints.agentic.unitree_go2_agentic_huggingface:unitree_go2_agentic_huggingface",
    "unitree-go2-agentic-mcp": "dimos.robot.unitree.go2.blueprints.agentic.unitree_go2_agentic_mcp:unitree_go2_agentic_mcp",
    "unitree-go2-agentic-ollama": "dimos.robot.unitree.go2.blueprints.agentic.unitree_go2_agentic_ollama:unitree_go2_agentic_ollama",
    "unitree-go2-basic": "dimos.robot.unitree.go2.blueprints.basic.unitree_go2_basic:unitree_go2_basic",
    "unitree-go2-detection": "dimos.robot.unitree.go2.blueprints.smart.unitree_go2_detection:unitree_go2_detection",
    "unitree-go2-ros": "dimos.robot.unitree.go2.blueprints.smart.unitree_go2_ros:unitree_go2_ros",
    "unitree-go2-spatial": "dimos.robot.unitree.go2.blueprints.smart.unitree_go2_spatial:unitree_go2_spatial",
    "unitree-go2-temporal-memory": "dimos.robot.unitree.go2.blueprints.agentic.unitree_go2_temporal_memory:unitree_go2_temporal_memory",
    "unitree-go2-vlm-stream-test": "dimos.robot.unitree.go2.blueprints.smart.unitree_go2_vlm_stream_test:unitree_go2_vlm_stream_test",
    "xarm-perception": "dimos.manipulation.manipulation_blueprints:xarm_perception",
    "xarm-perception-agent": "dimos.manipulation.manipulation_blueprints:xarm_perception_agent",
    "xarm6-planner-only": "dimos.manipulation.manipulation_blueprints:xarm6_planner_only",
    "xarm7-planner-coordinator": "dimos.manipulation.manipulation_blueprints:xarm7_planner_coordinator",
    "xarm7-trajectory-sim": "dimos.simulation.sim_blueprints:xarm7_trajectory_sim",
}


all_modules = {
    "agent": "dimos.agents.agent",
    "arm_teleop_module": "dimos.teleop.quest.quest_extensions",
    "camera_module": "dimos.hardware.sensors.camera.module",
    "cartesian_motion_controller": "dimos.manipulation.control.servo_control.cartesian_motion_controller",
    "control_coordinator": "dimos.control.coordinator",
    "cost_mapper": "dimos.mapping.costmapper",
    "demo_calculator_skill": "dimos.agents.skills.demo_calculator_skill",
    "demo_robot": "dimos.agents.skills.demo_robot",
    "depth_module": "dimos.robot.unitree.depth_module",
    "detection3d_module": "dimos.perception.detection.module3D",
    "detection_db_module": "dimos.perception.detection.moduleDB",
    "fastlio2_module": "dimos.hardware.sensors.lidar.fastlio2.module",
    "foxglove_bridge": "dimos.robot.foxglove_bridge",
    "g1_connection": "dimos.robot.unitree.g1.connection",
    "g1_sim_connection": "dimos.robot.unitree.g1.sim",
    "g1_skills": "dimos.robot.unitree.g1.skill_container",
    "go2_connection": "dimos.robot.unitree.go2.connection",
    "google_maps_skill": "dimos.agents.skills.google_maps_skill_container",
    "gps_nav_skill": "dimos.agents.skills.gps_nav_skill",
    "grasping_module": "dimos.manipulation.grasping.grasping",
    "joint_trajectory_controller": "dimos.manipulation.control.trajectory_controller.joint_trajectory_controller",
    "keyboard_teleop": "dimos.robot.unitree.keyboard_teleop",
    "keyboard_teleop_module": "dimos.teleop.keyboard.keyboard_teleop_module",
    "manipulation_module": "dimos.manipulation.manipulation_module",
    "mapper": "dimos.robot.unitree.type.map",
    "mid360_module": "dimos.hardware.sensors.lidar.livox.module",
    "navigation_skill": "dimos.agents.skills.navigation",
    "object_scene_registration_module": "dimos.perception.object_scene_registration",
    "object_tracking": "dimos.perception.object_tracker",
    "osm_skill": "dimos.agents.skills.osm",
    "person_follow_skill": "dimos.agents.skills.person_follow",
    "person_tracker_module": "dimos.perception.detection.person_tracker",
    "phone_teleop_module": "dimos.teleop.phone.phone_teleop_module",
    "quest_teleop_module": "dimos.teleop.quest.quest_teleop_module",
    "realsense_camera": "dimos.hardware.sensors.camera.realsense.camera",
    "replanning_a_star_planner": "dimos.navigation.replanning_a_star.module",
    "rerun_bridge": "dimos.visualization.rerun.bridge",
    "ros_nav": "dimos.navigation.rosnav",
    "simple_phone_teleop_module": "dimos.teleop.phone.phone_extensions",
    "simulation": "dimos.simulation.manipulators.sim_module",
    "spatial_memory": "dimos.perception.spatial_perception",
    "speak_skill": "dimos.agents.skills.speak_skill",
    "temporal_memory": "dimos.perception.experimental.temporal_memory.temporal_memory",
    "twist_teleop_module": "dimos.teleop.quest.quest_extensions",
    "unitree_skills": "dimos.robot.unitree.unitree_skill_container",
    "utilization": "dimos.utils.monitoring",
    "visualizing_teleop_module": "dimos.teleop.quest.quest_extensions",
    "vlm_agent": "dimos.agents.vlm_agent",
    "vlm_stream_tester": "dimos.agents.vlm_stream_tester",
    "voxel_mapper": "dimos.mapping.voxels",
    "wavefront_frontier_explorer": "dimos.navigation.frontier_exploration.wavefront_frontier_goal_selector",
    "web_input": "dimos.agents.web_human_input",
    "websocket_vis": "dimos.web.websocket_vis.websocket_vis_module",
    "zed_camera": "dimos.hardware.sensors.camera.zed.camera",
}
